# Copyright (C) 2019 Jan Zarsky, <jzarsky@redhat.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os
from unittest.mock import patch

import tests.test_main
import tests.selinux as mocked_selinux
import tests.semanage as mocked_semanage


class TestUnit(tests.test_main.TestBase):
    """Test basic functionality of udica"""

    def setUp(self):
        self.selinux_patch = patch.dict("sys.modules", selinux=mocked_selinux)
        self.selinux_patch.start()

        self.semanage_patch = patch.dict("sys.modules", semanage=mocked_semanage)
        self.semanage_patch.start()

        super().setUp()

    def tearDown(self):
        self.selinux_patch.stop()
        self.semanage_patch.stop()
        super().tearDown()



# Copyright (C) 2019 Jan Zarsky, <jzarsky@redhat.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from unittest.mock import Mock

semanage_handle_create = Mock()
semanage_connect = Mock()
semanage_context_get_type = Mock(side_effect=lambda x: x.split(":")[2])

ports = [
    ("system_u:object_r:inetd_child_port_t:s0", "tcp", 1, 1),
    ("system_u:object_r:inetd_child_port_t:s0", "tcp", 1, 1),
    ("system_u:object_r:echo_port_t:s0", "tcp", 7, 7),
    ("system_u:object_r:echo_port_t:s0", "tcp", 7, 7),
    ("system_u:object_r:inetd_child_port_t:s0", "tcp", 9, 9),
    ("system_u:object_r:inetd_child_port_t:s0", "tcp", 9, 9),
    ("system_u:object_r:inetd_child_port_t:s0", "tcp", 13, 13),
    # default ranges
    ("system_u:object_r:reserved_port_t:s0", "udp", 1, 511),
    ("system_u:object_r:reserved_port_t:s0", "tcp", 1, 511),
    ("system_u:object_r:reserved_port_t:s0", "sctp", 1, 511),
    ("system_u:object_r:hi_reserved_port_t:s0", "udp", 512, 1023),
    ("system_u:object_r:hi_reserved_port_t:s0", "tcp", 512, 1023),
    ("system_u:object_r:hi_reserved_port_t:s0", "sctp", 512, 1023),
    ("system_u:object_r:unreserved_port_t:s0", "udp", 61001, 65535),
    ("system_u:object_r:unreserved_port_t:s0", "tcp", 61001, 65535),
    ("system_u:object_r:ephemeral_port_t:s0", "udp", 32768, 60999),
    ("system_u:object_r:ephemeral_port_t:s0", "tcp", 32768, 60999),
    ("system_u:object_r:unreserved_port_t:s0", "udp", 1024, 32767),
    ("system_u:object_r:unreserved_port_t:s0", "tcp", 1024, 32767),
    ("system_u:object_r:unreserved_port_t:s0", "sctp", 1024, 65535),
]
ports_local = [
    ("system_u:object_r:inetd_child_port_t:s0", "tcp", 13, 13),
    ("system_u:object_r:inetd_child_port_t:s0", "tcp", 19, 19),
    ("system_u:object_r:inetd_child_port_t:s0", "tcp", 19, 19),
    ("system_u:object_r:ftp_data_port_t:s0", "tcp", 20, 20),
    ("system_u:object_r:ftp_port_t:s0", "tcp", 21, 21),
    ("system_u:object_r:mysqld_port_t:s0", "tcp", 63132, 63164),
]
semanage_port_list = Mock(return_value=(0, ports))
semanage_port_list_local = Mock(return_value=(0, ports_local))
semanage_port_get_con = Mock(side_effect=lambda x: x[0])
semanage_port_get_proto = Mock(side_effect=lambda x: x[1])
semanage_port_get_proto_str = Mock(side_effect=lambda x: x)
semanage_port_get_low = Mock(side_effect=lambda x: x[2])
semanage_port_get_high = Mock(side_effect=lambda x: x[3])

fcontexts = [
    ("/var/spool(/.*)?", "system_u:object_r:var_spool_t:s0"),
    ("/var/spool/[mg]dm(/.*)?", "system_u:object_r:xdm_spool_t:s0"),
    ("/var/spool/(client)?mqueue(/.*)?", "system_u:object_r:mqueue_spool_t:s0"),
    ("/var/spool/(.*/)?a?quota\\.(user|group)", "system_u:object_r:quota_db_t:s0"),
    ("/var/spool/at(/.*)?", "system_u:object_r:user_cron_spool_t:s0"),
    ("/var/spool/faf(/.*)?", "system_u:object_r:abrt_retrace_spool_t:s0"),
    ("/var/spool/fax(/.*)?", "system_u:object_r:getty_var_run_t:s0"),
    ("/var/spool/lpd(/.*)?", "system_u:object_r:print_spool_t:s0"),
    ("/var/spool/sms(/.*)?", "system_u:object_r:smsd_spool_t:s0"),
    ("/var/spool/abrt(/.*)?", "system_u:object_r:abrt_var_cache_t:s0"),
    ("/var/spool/ctdb(/.*)?", "system_u:object_r:ctdbd_spool_t:s0"),
    ("/var/spool/cups(/.*)?", "system_u:object_r:print_spool_t:s0"),
    ("/var/spool/gosa(/.*)?", "system_u:object_r:httpd_sys_rw_content_t:s0"),
    ("/var/spool/imap(/.*)?", "system_u:object_r:mail_spool_t:s0"),
    ("/var/spool/mail(/.*)?", "system_u:object_r:mail_spool_t:s0"),
    ("/var/spool/news(/.*)?", "system_u:object_r:news_spool_t:s0"),
    ("/var/spool/rwho(/.*)?", "system_u:object_r:rwho_spool_t:s0"),
    ("/var/spool/uucp(/.*)?", "system_u:object_r:uucpd_spool_t:s0"),
    ("/var/spool/exim[0-9]?(/.*)?", "system_u:object_r:exim_spool_t:s0"),
    ("/var/spool/audit(/.*)?", "system_u:object_r:audit_spool_t:s0"),
    ("/var/spool/debug(/.*)?", "system_u:object_r:abrt_var_cache_t:s0"),
    ("/var/spool/samba(/.*)?", "system_u:object_r:samba_spool_t:s0"),
    ("/var/spool/smtpd(/.*)?", "system_u:object_r:mail_spool_t:s0"),
    ("/var/spool/spamd(/.*)?", "system_u:object_r:spamd_spool_t:s0"),
    ("/var/spool/squid(/.*)?", "system_u:object_r:squid_cache_t:s0"),
    ("/var/spool/texmf(/.*)?", "system_u:object_r:tetex_data_t:s0"),
    ("/var/spool/voice(/.*)?", "system_u:object_r:getty_var_run_t:s0"),
    ("/var/spool/bacula.*", "system_u:object_r:bacula_spool_t:s0"),
    ("/var/spool/icinga(/.*)?", "system_u:object_r:nagios_spool_t:s0"),
    ("/var/spool/nagios(/.*)?", "system_u:object_r:nagios_spool_t:s0"),
    ("/var/spool/snmptt(/.*)?", "system_u:object_r:snmpd_var_lib_t:s0"),
    ("/var/spool/spampd(/.*)?", "system_u:object_r:spamd_spool_t:s0"),
    ("/var/spool/viewvc(/.*)?", "system_u:object_r:httpd_sys_rw_content_t:s0"),
    ("/var/spool/cron/a?quota\\.(user|group)", "system_u:object_r:quota_db_t:s0"),
    ("/var/spool/mailman.*", "system_u:object_r:mailman_data_t:s0"),
    ("/var/spool/postfix.*", "system_u:object_r:postfix_spool_t:s0"),
    ("/var/spool/amavisd(/.*)?", "system_u:object_r:antivirus_db_t:s0"),
    ("/var/spool/anacron(/.*)?", "system_u:object_r:system_cron_spool_t:s0"),
    ("/var/spool/courier(/.*)?", "system_u:object_r:courier_spool_t:s0"),
    ("/var/spool/dovecot(/.*)?", "system_u:object_r:dovecot_spool_t:s0"),
    ("/var/spool/prelude(/.*)?", "system_u:object_r:prelude_spool_t:s0"),
    ("/var/spool/pyicq-t(/.*)?", "system_u:object_r:pyicqt_var_spool_t:s0"),
    ("/var/spool/rsyslog(/.*)?", "system_u:object_r:var_log_t:s0"),
    ("/var/spool/up2date(/.*)?", "system_u:object_r:rpm_var_cache_t:s0"),
    ("/var/spool/asterisk(/.*)?", "system_u:object_r:asterisk_spool_t:s0"),
    ("/var/spool/cups-pdf(/.*)?", "system_u:object_r:print_spool_t:s0"),
    ("/var/spool/opendkim(/.*)?", "system_u:object_r:dkim_milter_data_t:s0"),
    ("/var/spool/plymouth(/.*)?", "system_u:object_r:plymouthd_spool_t:s0"),
    ("/var/spool/mqueue\\.in(/.*)?", "system_u:object_r:mqueue_spool_t:s0"),
    ("/var/spool/opendmarc(/.*)?", "system_u:object_r:dkim_milter_data_t:s0"),
    ("/var/spool/MIMEDefang(/.*)?", "system_u:object_r:spamd_var_run_t:s0"),
    ("/var/spool/authdaemon(/.*)?", "system_u:object_r:courier_spool_t:s0"),
    ("/var/spool/bacula/log(/.*)?", "system_u:object_r:var_log_t:s0"),
    ("/var/spool/callweaver(/.*)?", "system_u:object_r:callweaver_spool_t:s0"),
    ("/var/spool/gridengine(/.*)?", "system_u:object_r:sge_spool_t:s0"),
    ("/var/spool/rhsm/debug(/.*)?", "system_u:object_r:abrt_var_cache_t:s0"),
    ("/var/spool/turboprint(/.*)?", "system_u:object_r:lpd_var_run_t:s0"),
    ("/var/spool/uucppublic(/.*)?", "system_u:object_r:uucpd_spool_t:s0"),
    ("/var/spool/MailScanner(/.*)?", "system_u:object_r:mscan_spool_t:s0"),
    ("/var/spool/abrt-upload(/.*)?", "system_u:object_r:public_content_rw_t:s0"),
    ("/var/spool/postfix/etc(/.*)?", "system_u:object_r:etc_t:s0"),
    ("/var/spool/postfix/lib(/.*)?", "system_u:object_r:lib_t:s0"),
    ("/var/spool/postfix/usr(/.*)?", "system_u:object_r:lib_t:s0"),
    ("/var/spool/postfix/pid/.*", "system_u:object_r:postfix_var_run_t:s0"),
    ("/var/spool/abrt-retrace(/.*)?", "system_u:object_r:abrt_retrace_spool_t:s0"),
    ("/var/spool/milter-regex(/.*)?", "system_u:object_r:regex_milter_data_t:s0"),
    ("/var/spool/spamassassin(/.*)?", "system_u:object_r:spamd_spool_t:s0"),
    ("/var/spool/squirrelmail(/.*)?", "system_u:object_r:squirrelmail_spool_t:s0"),
    ("/var/spool/MD-Quarantine(/.*)?", "system_u:object_r:spamd_var_run_t:s0"),
    ("/var/spool/postfix/defer(/.*)?", "system_u:object_r:postfix_spool_t:s0"),
    ("/var/spool/postfix/flush(/.*)?", "system_u:object_r:postfix_spool_t:s0"),
    ("/var/spool/postfix/lib64(/.*)?", "system_u:object_r:lib_t:s0"),
    ("/var/spool/postfix/bounce(/.*)?", "system_u:object_r:postfix_spool_bounce_t:s0"),
    ("/var/spool/postfix/public(/.*)?", "system_u:object_r:postfix_public_t:s0"),
    ("/var/spool/retrace-server(/.*)?", "system_u:object_r:abrt_retrace_spool_t:s0"),
    ("/var/spool/postfix/lib/ld.*\\.so.*", "system_u:object_r:ld_so_t:s0"),
    ("/var/spool/postfix/private(/.*)?", "system_u:object_r:postfix_private_t:s0"),
    ("/var/spool/postfix/spamass(/.*)?", "system_u:object_r:spamass_milter_data_t:s0"),
    ("/var/spool/prelude-manager(/.*)?", "system_u:object_r:prelude_spool_t:s0"),
    ("/var/spool/postfix/deferred(/.*)?", "system_u:object_r:postfix_spool_t:s0"),
    ("/var/spool/postfix/maildrop(/.*)?", "system_u:object_r:postfix_spool_t:s0"),
    ("/var/spool/postfix/postgrey(/.*)?", "system_u:object_r:postgrey_spool_t:s0"),
    ("/var/spool/zoneminder-upload(/.*)?", "system_u:object_r:zoneminder_spool_t:s0"),
]
fcontexts_local = [
    ("/var/spool/cron", "system_u:object_r:user_cron_spool_t:s0"),
    ("/var/spool/fcron", "system_u:object_r:cron_spool_t:s0"),
    ("/var/spool/postfix/dev", "system_u:object_r:device_t:s0"),
    ("/var/spool/postfix/pid", "system_u:object_r:var_run_t:s0"),
    ("/var/spool/fcron/systab", "system_u:object_r:system_cron_spool_t:s0"),
]
fcontexts_homedirs = [
    ("/var/spool/cron/crontabs", "system_u:object_r:cron_spool_t:s0"),
    ("/var/spool/postfix/dev/log", "system_u:object_r:devlog_t:s0"),
    ("/var/spool/fcron/new\\.systab", "system_u:object_r:system_cron_spool_t:s0"),
    ("/var/spool/fcron/systab\\.orig", "system_u:object_r:system_cron_spool_t:s0"),
    ("/var/spool/postfix/etc/localtime", "system_u:object_r:locale_t:s0"),
    ("/var/spool/cron/user", "system_u:object_r:user_cron_spool_t:s0"),
]

semanage_fcontext_list = Mock(return_value=(0, fcontexts))
semanage_fcontext_list_local = Mock(return_value=(0, fcontexts_local))
semanage_fcontext_list_homedirs = Mock(return_value=(0, fcontexts_homedirs))
semanage_fcontext_get_expr = Mock(side_effect=lambda x: x[0])
semanage_fcontext_get_con = Mock(side_effect=lambda x: x[1])



# Copyright (C) 2019 Jan Zarsky, <jzarsky@redhat.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import os
import unittest
from unittest.mock import patch

# Import tarfile library to extract tarball with udica policy and templates when --ansible
# parameter is used
import tarfile

# Use the selinux and semanage packages provided by the system instead of the mock ones. When
# running on a system with SELinux disabled (e.g. in a container), it must be set to False.
# On RHEL, CentOS or Fedora it may be set to True.
SELINUX_ENABLED = False

# Globally define symbol "udica" so we can override the import
udica = None

TEST_DIR_PATH = os.path.dirname(os.path.abspath(__file__))


def test_file(path):
    return os.path.join(TEST_DIR_PATH, path)


class TestBase(unittest.TestCase):
    """Test basic functionality of udica"""

    def setUp(self):
        # NOTE: We import udica here so the above mocked modules take place.
        global udica
        udica = __import__("udica.__main__")

        # Overwrite paths to files so that they do not have to be installed.
        udica.policy.TEMPLATE_PLAYBOOK = os.path.join(
            TEST_DIR_PATH, "../", "udica/ansible/deploy-module.yml"
        )
        udica.policy.TEMPLATES_STORE = os.path.join(
            TEST_DIR_PATH, "../", "udica/templates"
        )
        # FIXME: the policy module is using global variable which must be reset to []
        udica.policy.templates_to_load = []

    def tearDown(self):
        os.unlink("my_container.cil")

        global udica
        udica = None

    def test_basic_podman(self):
        """podman run -v /home:/home:ro -v /var/spool:/var/spool:rw -p 21:21 fedora"""
        output = self.run_udica(
            ["udica", "-j", "tests/test_basic.podman.json", "my_container"]
        )
        self.assert_templates(
            output, ["base_container", "net_container", "home_container"]
        )
        self.assert_policy(test_file("test_basic.podman.cil"))

    def test_basic_oci(self):
        """podman run -v /home:/home:ro -v /var/spool:/var/spool:rw -p 21:21 ubi8"""
        output = self.run_udica(
            ["udica", "-j", "tests/test_basic.oci.json", "my_container"]
        )
        self.assert_templates(
            output, ["base_container", "net_container", "home_container"]
        )
        self.assert_policy(test_file("test_basic.podman.cil"))

    def test_basic_docker(self):
        """docker run -v /home:/home:ro -v /var/spool:/var/spool:rw -p 21:21 fedora"""
        output = self.run_udica(
            ["udica", "-j", "tests/test_basic.docker.json", "my_container"]
        )
        self.assert_templates(
            output, ["base_container", "net_container", "home_container"]
        )
        self.assert_policy(test_file("test_basic.docker.cil"))

    def test_basic_cri(self):
        """Start CRI-O mounting /var/spool with read/write perms and /home with readonly perms"""
        output = self.run_udica(
            [
                "udica",
                "-j",
                "tests/test_basic.cri.json",
                "--full-network-access",
                "my_container",
            ]
        )
        self.assert_templates(
            output, ["base_container", "net_container", "home_container"]
        )
        self.assert_policy(test_file("test_basic.cri.cil"))

    def test_basic_specified_engine_cri(self):
        """Start CRI-O mounting /var/spool with read/write perms and /home with readonly perms"""
        output = self.run_udica(
            [
                "udica",
                "--container-engine",
                "CRI-O",
                "-j",
                "tests/test_basic.cri.json",
                "--full-network-access",
                "my_container",
            ]
        )
        self.assert_templates(
            output, ["base_container", "net_container", "home_container"]
        )
        self.assert_policy(test_file("test_basic.cri.cil"))

    def test_default_podman(self):
        """podman run fedora"""
        output = self.run_udica(
            ["udica", "-j", "tests/test_default.podman.json", "my_container"]
        )
        self.assert_templates(output, ["base_container"])
        self.assert_policy(test_file("test_default.podman.cil"))

    def test_default_specified_engine_podman(self):
        """podman run fedora"""
        output = self.run_udica(
            [
                "udica",
                "-e",
                "podman",
                "-j",
                "tests/test_default.podman.json",
                "my_container",
            ]
        )
        self.assert_templates(output, ["base_container"])
        self.assert_policy(test_file("test_default.podman.cil"))

    def test_default_docker(self):
        """docker run fedora"""
        output = self.run_udica(
            ["udica", "-j", "tests/test_default.docker.json", "my_container"]
        )
        self.assert_templates(output, ["base_container"])
        self.assert_policy(test_file("test_default.docker.cil"))

    def test_default_specified_engine_docker(self):
        """docker run fedora"""
        output = self.run_udica(
            [
                "udica",
                "-e",
                "docker",
                "-j",
                "tests/test_default.docker.json",
                "my_container",
            ]
        )
        self.assert_templates(output, ["base_container"])
        self.assert_policy(test_file("test_default.docker.cil"))

    def test_default_oci(self):
        """podman run ubi8"""
        output = self.run_udica(
            ["udica", "-j", "tests/test_default.oci.json", "my_container"]
        )
        self.assert_templates(output, ["base_container"])
        self.assert_policy(test_file("test_default.podman.cil"))

    def test_port_ranges_podman(self):
        """podman run -p 63140:63140 fedora"""
        output = self.run_udica(
            ["udica", "-j", "tests/test_ports.podman.json", "my_container"]
        )
        self.assert_templates(output, ["base_container", "net_container"])
        self.assert_policy(test_file("test_ports.podman.cil"))

    def test_port_ranges_docker(self):
        """docker run -p 63140:63140 fedora"""
        output = self.run_udica(
            ["udica", "-j", "tests/test_ports.docker.json", "my_container"]
        )
        self.assert_templates(output, ["base_container", "net_container"])
        self.assert_policy(test_file("test_ports.docker.cil"))

    def test_default_ansible_podman(self):
        """podman run fedora"""
        output = self.run_udica(
            [
                "udica",
                "-j",
                "tests/test_default.podman.json",
                "my_container",
                "--ansible",
            ]
        )
        self.assert_ansible(
            output, ["base_container"], test_file("test_default.ansible.podman.yml")
        )
        self.assert_policy(test_file("test_default.podman.cil"))

    def test_basic_ansible_podman(self):
        """podman run -v /home:/home:ro -v /var/spool:/var/spool:rw -p 21:21 fedora"""
        output = self.run_udica(
            ["udica", "-j", "tests/test_basic.podman.json", "my_container", "--ansible"]
        )
        self.assert_ansible(
            output,
            ["base_container", "net_container", "home_container"],
            test_file("test_basic.ansible.podman.yml"),
        )
        self.assert_policy(test_file("test_basic.podman.cil"))

    def test_nocontext_podman(self):
        """podman run -v /tmp/test:/tmp/test:rw fedora"""
        os.makedirs("/tmp/test", exist_ok=True)
        output = self.run_udica(
            ["udica", "-j", "tests/test_nocontext.podman.json", "my_container"]
        )
        self.assert_templates(output, ["base_container"])
        self.assert_policy(test_file("test_nocontext.podman.cil"))
        os.rmdir("/tmp/test")

    def test_stream_connect_podman(self):
        """podman run fedora"""
        output = self.run_udica(
            [
                "udica",
                "-j",
                "tests/test_default.podman.json",
                "--stream-connect",
                "network_container",
                "my_container",
            ]
        )
        self.assert_templates(output, ["base_container"])
        self.assert_policy(test_file("test_stream_connect.podman.cil"))

    def test_fullnetworkaccess_podman(self):
        """podman run fedora"""
        output = self.run_udica(
            [
                "udica",
                "-j",
                "tests/test_default.podman.json",
                "--full-network-access",
                "my_container",
            ]
        )
        self.assert_templates(output, ["base_container", "net_container"])
        self.assert_policy(test_file("test_fullnetworkaccess.podman.cil"))

    def test_virtaccess_podman(self):
        """podman run fedora"""
        output = self.run_udica(
            [
                "udica",
                "-j",
                "tests/test_default.podman.json",
                "--virt-access",
                "my_container",
            ]
        )
        self.assert_templates(output, ["base_container", "virt_container"])
        self.assert_policy(test_file("test_virtaccess.podman.cil"))

    def test_xaccess_podman(self):
        """podman run fedora"""
        output = self.run_udica(
            [
                "udica",
                "-j",
                "tests/test_default.podman.json",
                "--X-access",
                "my_container",
            ]
        )
        self.assert_templates(output, ["base_container", "x_container"])
        self.assert_policy(test_file("test_xaccess.podman.cil"))

    def test_ttyaccess_podman(self):
        """podman run fedora"""
        output = self.run_udica(
            [
                "udica",
                "-j",
                "tests/test_default.podman.json",
                "--tty-access",
                "my_container",
            ]
        )
        self.assert_templates(output, ["base_container", "tty_container"])
        self.assert_policy(test_file("test_ttyaccess.podman.cil"))

    def test_append_more_rules_podman(self):
        """podman run fedora"""
        output = self.run_udica(
            [
                "udica",
                "-j",
                "tests/test_default.podman.json",
                "-a",
                "tests/append_avc_file",
                "my_container",
            ]
        )
        self.assert_templates(output, ["base_container"])
        self.assert_policy(test_file("test_append_avc.podman.cil"))

    def run_udica(self, args):
        with patch("sys.argv", args):
            with patch("sys.stderr.write") as mock_err, patch(
                "sys.stdout.write"
            ) as mock_out:
                mock_out.output = ""

                def store_output(output):
                    mock_out.output += output

                mock_out.side_effect = store_output
                udica.__main__.main()
                mock_err.assert_not_called()

        self.assertRegex(mock_out.output, "Policy my_container created")
        self.assertRegex(
            mock_out.output, "--security-opt label=type:my_container.process"
        )

        return mock_out.output

    def assert_templates(self, output, templates):
        self.assertRegex(output, "semodule -i my_container")
        if templates:
            if len(templates) > 1:
                templ_str = "{%s.cil}" % ".cil,".join(templates)
            else:
                templ_str = templates[0] + ".cil"
            self.assertRegex(output, udica.policy.TEMPLATES_STORE + "/" + templ_str)

    def assert_policy(self, policy_file):
        self.assertTrue(os.path.isfile("my_container.cil"))
        with open("my_container.cil") as cont:
            policy = cont.read().strip()
        with open(policy_file) as cont:
            exp_policy = cont.read().strip()
        self.assertMultiLineEqual(policy, exp_policy)

    def assert_ansible(self, output, templates, variables_file):
        udica.policy.TEMPLATES_STORE = "./"
        self.assertRegex(
            output, "Ansible playbook and archive with udica policies generated!"
        )
        with tarfile.open("my_container-policy.tar.gz") as archive:
            archive.extractall()

        self.assertTrue(os.path.isfile("deploy-module.yml"))

        with open("deploy-module.yml") as cont:
            playbook = cont.read().strip()
        with open(udica.policy.TEMPLATE_PLAYBOOK) as cont:
            exp_playbook = cont.read().strip()
        self.assertMultiLineEqual(playbook, exp_playbook)

        self.assertTrue(os.path.isfile("variables-deploy-module.yml"))

        with open("variables-deploy-module.yml") as cont:
            variables_playbook = cont.read().strip()
        with open(variables_file) as cont:
            exp_variables_playbook = cont.read().strip()
        self.assertMultiLineEqual(variables_playbook, exp_variables_playbook)

        os.unlink("variables-deploy-module.yml")
        os.unlink("my_container-policy.tar.gz")
        os.unlink("deploy-module.yml")

        for temp in templates:
            os.unlink(temp + ".cil")



# Copyright (C) 2019 Jan Zarsky, <jzarsky@redhat.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from unittest.mock import Mock

SELABEL_CTX_FILE = None

selabel_open = Mock()


def selabel_lookup(selabel, directory, rc):
    if directory == "/tmp/test":
        return (0, None)
    else:
        return (0, "system_u:object_r:var_spool_t:s0")


def getfilecon(directory):
    if directory == "/tmp/test":
        return (0, "system_u:object_r:user_tmp_t:s0")
    else:
        return (0, "system_u:object_r:var_spool_t:s0")
